/** @file infixTOpostfix.c
 * @brief Converting from infix to postfix.
 *
 * Read in the tokens one at a time. If it is an operator, push it on the stack;
 * if it is an integer, print it out; if it is a right parentheses, pop the 
 * topmost element from the stack and print it out; if it is a left parentheses,
 * ignore it.  
 *
 * @author Pavlos Antoniou
 * @bug No known bugs
 */
 
#include <stdio.h>
#include <stdlib.h>

/*
 * The stack structure
 */
typedef struct stack {
  int value;
  struct stack *prev;
} stack ;


/** @brief Push an element to the stack
 *
 * @param stack** the stack
 * @param int the element
 * @return void
 */
void push(stack **, int);


/** @brief Pop value from the stack
 *
 * @param stack** the stack
 * @return int the element popped from stack
 */
int pop(stack **);


/** @brief Clear the stack
 *
 * Deletes the elements of the stack. Free the memory.
 *
 * @param stack** the stack
 * @return void
 */
void clear(stack **);


/** @brief Returns the number of elements in the stack
 *
 * @param stack* the stack
 * @return int number of elements
 */
int length(stack *p);


/** @brief Displays the stack
 *
 * @param stack* the stack
 * @return void
 */
void display(stack *);
           

/*************************************************************************/

void push(stack **p, int val) {
  stack *element = (stack *)malloc(sizeof(stack));
  if(!element) 
    exit(EXIT_FAILURE);   
  
  element->value = val;
  element->prev = *p;
  *p = element;       
}
/*************************************************************************/

int pop(stack **p) {
  int val;
  stack *tmp;
  if(!*p) return -1;     /* Retourne -1 si la stack est vide. */
  tmp = (*p)->prev;
  val = (*p)->value;
  free(*p);
  *p = tmp;       /* Le pointeur pointe sur le dernier élément. */
  return val;     /* Retourne la value soutirée de la stack. */
}

/*************************************************************************/

void clear(stack **p) {
  stack *tmp;
  while(*p) {
    tmp = (*p)->prev;
    free(*p);
    *p = tmp;
  }
}
/*************************************************************************/

int length(stack *p) {
  int n=0;
  while(p) {
    n++;
    p = p->prev;
  }
  return n;
}

/*************************************************************************/

void display(stack *p) {
  while(p) {
    printf("%d\n",p->value);
    p = p->prev;
  }
}


int main (int argc, char **argv) {

  char c;

  char PLUS     = '+';
  char MINUS    = '-';
  char TIMES    = '*';
  char DIVIS    = '/';

  stack *parStack;
  parStack = NULL;

  printf("Enter an infix expression : ");

  while ((c = getchar()) != '\n') {

    if (c == PLUS)
      push(&parStack, c);
    else 
      if (c == TIMES) 
	push(&parStack, c);
      else 
	if (c == MINUS) 
	  push(&parStack, c);
	else
	  if (c == DIVIS) 
	    push(&parStack, c);
	  else
	    if (c == ')') 
	      printf("%c ", pop(&parStack)); 
	    else
	      if (c == '(') 
		printf("");
	      else  
		printf("%c ", c);
  }
  printf("\n");
  
  return 0;

}

